/***************************************************************
 * Name:      wx013Main.cpp
 * Purpose:   Code for Application Frame
 * Author:    Salvador Pozo (salvador@conclase.net)
 * Created:   2024-12-18
 * Copyright: Salvador Pozo (https://conclase.net)
 * License:
 **************************************************************/

#ifdef WX_PRECOMP
#include "wx_pch.h"
#endif

#include "wx013Main.h"

//helper functions
enum wxbuildinfoformat {
    short_f, long_f };

wxString wxbuildinfo(wxbuildinfoformat format)
{
    wxString wxbuild(wxVERSION_STRING);

    if (format == long_f )
    {
#if defined(__WXMSW__)
        wxbuild << _T("-Windows");
#elif defined(__WXMAC__)
        wxbuild << _T("-Mac");
#elif defined(__UNIX__)
        wxbuild << _T("-Linux");
#endif

#if wxUSE_UNICODE
        wxbuild << _T("-Unicode build");
#else
        wxbuild << _T("-ANSI build");
#endif // wxUSE_UNICODE
    }

    return wxbuild;
}

BEGIN_EVENT_TABLE(wx013Frame, wxFrame)
    EVT_CLOSE(wx013Frame::OnClose)
    EVT_MENU(idMenuQuit, wx013Frame::OnQuit)
    EVT_MENU(idMenuAbout, wx013Frame::OnAbout)
    EVT_PAINT(wx013Frame::OnPaint)
END_EVENT_TABLE()

wx013Frame::wx013Frame(wxFrame *frame, const wxString& title)
    : wxFrame(frame, -1, title, wxDefaultPosition, wxSize(wxDefaultCoord, 330))
{
    // create a menu bar
    wxMenuBar* mbar = new wxMenuBar();
    wxMenu* fileMenu = new wxMenu(_T(""));
    fileMenu->Append(idMenuQuit, _T("&Salir\tAlt-F4"), _T("Salir de la aplicación"));
    mbar->Append(fileMenu, _T("&Fichero"));

    wxMenu* helpMenu = new wxMenu(_T(""));
    helpMenu->Append(idMenuAbout, _T("&Sobre\tF1"), _T("Muestra información sobre la aplicación"));
    mbar->Append(helpMenu, _T("&Ayuda"));

    SetMenuBar(mbar);

    // create a status bar with some information about the used wxWidgets version
    CreateStatusBar(2);
    SetStatusText(_("Hola"),0);
    SetStatusText(wxbuildinfo(short_f), 1);

}


wx013Frame::~wx013Frame()
{
}

void wx013Frame::OnClose(wxCloseEvent &event)
{
    Destroy();
}

void wx013Frame::OnQuit(wxCommandEvent &event)
{
    Destroy();
}

void wx013Frame::OnAbout(wxCommandEvent &event)
{
    wxString msg = wxbuildinfo(long_f);
    wxMessageBox(msg, _("Bienvenido a ..."));
}

void wx013Frame::OnPaint(wxPaintEvent& event)
{
    wxString cad;
    wxPaintDC dc(this);
    wxDateTime date;
    wxDateSpan datespan;
    wxTimeSpan timespan;

    date.SetToCurrent();
    cad = date.Format(_T("Hoy: %A %d de %B de %Y"));
    dc.DrawText(cad, wxPoint(10,10));
    date.Add(datespan.Day());
    cad = date.Format(_T("Mañana: %A %d de %B de %Y"));
    dc.DrawText(cad, wxPoint(10,30));
    date.SetToCurrent();
    date.Add(datespan.Months(5));
    cad = date.Format(_T("En 5 meses: %A %d de %B de %Y"));
    dc.DrawText(cad, wxPoint(10,50));
    date.SetToCurrent();
    date.Subtract(datespan.Years(60));
    cad = date.Format(_T("Hace 60 años: %A %d de %B de %Y"));
    dc.DrawText(cad, wxPoint(10,70));
    date.SetToCurrent();
    date -= wxDateSpan(570, 5, 0, 3);
    cad = date.Format(_T("Hace 570 años, 5 meses y 3 días: %A %d de %B de %Y"));
    dc.DrawText(cad, wxPoint(10,90));

    date.SetToCurrent();
    date.Add(timespan.Hours(12));
    cad = date.Format(_T("En 12 horas: %A %d de %B de %Y %H:%M:%S"));
    dc.DrawText(cad, wxPoint(10,120));
    date.SetToCurrent();
    date.Add(timespan.Seconds(200000));
    cad = date.Format(_T("En 200000 segundos: %A %d de %B de %Y %H:%M:%S"));
    dc.DrawText(cad, wxPoint(10,140));

    wxDateTimeHolidayAuthority::AddAuthority(new wxDateTimeFestivos2024);

    date.Set(25,wxDateTime::Dec,2024,0,0,0,0);
    cad = date.Format(_T("Navidad 2024: %A %d de %B de %Y"));
    dc.DrawText(cad, wxPoint(10,170));
    if(date.IsWorkDay()) cad = "Laborable"; else cad = "Festivo";
    dc.DrawText(cad, wxPoint(10,190));

    date.Add(datespan.Day());
    cad = date.Format(_T("%A %d de %B de %Y"));
    dc.DrawText(cad, wxPoint(10,210));
    if(date.IsWorkDay()) cad = "Laborable"; else cad = "Festivo";
    dc.DrawText(cad, wxPoint(10,230));

}

bool wxDateTimeFestivos2024::DoIsHoliday(const wxDateTime& dt) const
{
    auto busca = festivo.find(dt);
    return busca != festivo.end();
}

size_t wxDateTimeFestivos2024::DoGetHolidaysInRange(const wxDateTime& dtStart,
                                                const wxDateTime& dtEnd,
                                                wxDateTimeArray& holidays) const
{
    if ( dtStart > dtEnd )
    {
        wxFAIL_MSG( wxT("rango de fechas inválido en GetHolidaysInRange") );

        return 0u;
    }

    holidays.Empty();

    for (wxDateTime dt : festivo) {
       if(dt >= dtStart && dt <= dtEnd) holidays.Add(dt);
    }

    return holidays.GetCount();
}
